#include <iostream>
#include <fstream>
#include <ImageStack/ImageStack.h>
#include <cstdio>
#include "LocalHistograms.h"
using namespace std;
using namespace ImageStack;

enum FilterType {
	LOCAL_MODE, GLOBAL_MODE, MEDIAN
};

FilterType type = LOCAL_MODE;
float sigmaIm = 5;
float sigmaHist = 0.06;
int nBins = 30;
int antialiasWidth = 3;
float medianTarget = 0.5;

void processIm(Image &im, Image &out) {
	out = im;
	
	if (type == LOCAL_MODE) {
		LocalHistograms hist(im, sigmaIm, sigmaHist, nBins, HISTOGRAM_DERIVATIVE);
		out = hist.localMode(antialiasWidth);
	} else if (type == MEDIAN) {
		LocalHistograms hist(im, sigmaIm, sigmaHist, nBins, HISTOGRAM_INTEGRAL);
		out = hist.median(medianTarget,antialiasWidth);
	} else if (type == GLOBAL_MODE) {
		LocalHistograms hist(im, sigmaIm, sigmaHist, nBins, (HistogramType)(HISTOGRAM_VALUE | HISTOGRAM_DERIVATIVE));
		out = hist.globalMode(antialiasWidth);
	}
}

int main(int argc, char *argv[]) {
	if (argc < 3) {
		cout << "Usage: " << argv[0] << " in.jpg out.jpg [color] [filter] [options]\n\n";
		cout << "Color space choices:\n";
		cout << "\t-rgb Filter RGB channels separately\n";
		cout << "\t-hsv Filter V of HSV channels (default)\n\n";
		cout << "Filters:\n";
		cout << "\t-localMode Local mode filter (default)\n";
		cout << "\t-median Median filter\n";
		cout << "\t-globalMode Global mode filter\n\n";
		cout << "Options:\n";
		cout << "\t-sigmaIm Sets standard deviation in image space (default = 5)\n";
		cout << "\t-sigmaHist Sets standard deviation in histogram space (default = 0.06)\n";
		cout << "\t-bins Sets number of histogram bins (default = 30)\n";
		cout << "\t-antialiasWidth Sets number of samples in n x n antialiasing grid (default = 3)\n";
		cout << "\t-medianTarget Parameter for morphological filtering (default = 0.5)\n";
		return 0;
	}
	
	Image im = Load::apply(argv[1]);
	
	bool hsv = true;
	for (int i = 3; i < argc; i++) {
		string argument = argv[i];
		if (argument == "-hsv") hsv = true;
		else if (argument == "-rgb") hsv = false;
		else if (argument == "-localMode") type = LOCAL_MODE;
		else if (argument == "-median") type = MEDIAN;
		else if (argument == "-globalMode") type = GLOBAL_MODE;
		else if (argument == "-sigmaIm") sscanf(argv[++i],"%g", &sigmaIm);
		else if (argument == "-sigmaHist") sscanf(argv[++i],"%g", &sigmaHist);
		else if (argument == "-bins") sscanf(argv[++i],"%d", &nBins);
		else if (argument == "-antialiasWidth") sscanf(argv[++i],"%d", &antialiasWidth);
		else if (argument == "-medianTarget") sscanf(argv[++i],"%g", &medianTarget);
	}
	
	Image toProcess(im.width,im.height,1,1), processed(im.width,im.height,1,1);
	
	if (hsv) {
		Image hsv = ColorConvert::rgb2hsv(im);
		Clamp::apply(hsv,0.000001,0.99999); // not great, but hsv->rgb breaks otherwise
		
		for (int y = 0; y < im.height; y++)
		for (int x = 0; x < im.width; x++) toProcess(x,y,0)[0] = hsv(x,y,0)[2];
		
		processIm(toProcess, processed);
		
		for (int y = 0; y < im.height; y++)
		for (int x = 0; x < im.width; x++) hsv(x,y,0)[2] = processed(x,y,0)[0];
		
		im = ColorConvert::hsv2rgb(hsv);
	} else { // do each channel separately
		for (int c = 0; c < 3; c++) {
			for (int y = 0; y < im.height; y++)
			for (int x = 0; x < im.width; x++) toProcess(x,y,0)[0] = im(x,y,0)[c];
			
			processIm(toProcess, processed);
			
			for (int y = 0; y < im.height; y++)
			for (int x = 0; x < im.width; x++) im(x,y,0)[c] = processed(x,y,0)[0];
		}
	}
	
	Save:: apply(im, argv[2]);
	
	return 0;
}
